package com.bipt.intelligentapplicationorchestrationservice.config;


import com.bipt.intelligentapplicationorchestrationservice.service.CacheManager;
import com.bipt.intelligentapplicationorchestrationservice.entity.entity.GpuResource;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionSynchronization;
import org.springframework.transaction.support.TransactionSynchronizationManager;

@Aspect
@Component
public class CacheAopConfig {
    private final CacheManager cacheManager;

    public CacheAopConfig(CacheManager cacheManager) {
        this.cacheManager = cacheManager;
    }

    // 定义写操作切点
    @Pointcut("@annotation(org.springframework.transaction.annotation.Transactional) && " +
            "execution(* com.bipt.intelligentapplicationorchestrationservice.service..*.*(..))")
    public void writeOperation() {}

    // 事务提交后操作
    @AfterReturning(pointcut = "writeOperation()", returning = "result")
    public void afterWriteCommit(JoinPoint joinPoint, Object result) {
        TransactionSynchronizationManager.registerSynchronization(
                new TransactionSynchronization() {
                    @Override
                    public void afterCommit() {
                        processCacheUpdate(result);
                    }
                });
    }

    private void processCacheUpdate(Object result) {
        if (result instanceof GpuResource) {
            GpuResource gpu = (GpuResource) result;
            cacheManager.refreshCache(gpu.getGPUId());
        } else if (result instanceof Long) { // 处理删除操作返回ID的情况
            cacheManager.evictCache((Long) result);
        }
    }
}